@@ -1,7 +1,9 @@
-//! Crds Gossip Pull overlay
+//! Crds Gossip Pull overlay.
+//!
 //! This module implements the anti-entropy protocol for the network.
 //!
 //! The basic strategy is as follows:
+//!
 //! 1. Construct a bloom filter of the local data set
 //! 2. Randomly ask a node on the network for data that is not contained in the bloom filter.
 //!
@@ -23,7 +25,7 @@ use {
     lru::LruCache,
     rand::Rng,
     rayon::{prelude::*, ThreadPool},
-    solana_bloom::bloom::{AtomicBloom, Bloom},
+    solana_runtime::bloom::{AtomicBloom, Bloom},
     solana_sdk::{
         hash::{hash, Hash},
         pubkey::Pubkey,
@@ -33,11 +35,11 @@ use {
     std::{
         collections::{HashMap, HashSet, VecDeque},
         convert::TryInto,
-        iter::repeat_with,
+        iter::{repeat, repeat_with},
         net::SocketAddr,
         sync::{
             atomic::{AtomicI64, AtomicUsize, Ordering},
-            Mutex,
+            Mutex, RwLock,
         },
         time::{Duration, Instant},
     },
@@ -186,36 +188,36 @@ pub struct ProcessPullStats {
 }
 
 pub struct CrdsGossipPull {
-    /// timestamp of last request
-    pub(crate) pull_request_time: LruCache<Pubkey, u64>,
+    /// Timestamp of last request
+    pull_request_time: RwLock<LruCache<Pubkey, /*timestamp:*/ u64>>,
     // Hash value and record time (ms) of the pull responses which failed to be
     // inserted in crds table; Preserved to stop the sender to send back the
     // same outdated payload again by adding them to the filter for the next
     // pull request.
-    pub failed_inserts: VecDeque<(Hash, u64)>,
+    failed_inserts: RwLock<VecDeque<(Hash, /*timestamp:*/ u64)>>,
     pub crds_timeout: u64,
-    pub msg_timeout: u64,
-    pub num_pulls: usize,
+    msg_timeout: u64,
+    pub num_pulls: AtomicUsize,
 }
 
 impl Default for CrdsGossipPull {
     fn default() -> Self {
         Self {
-            pull_request_time: LruCache::new(CRDS_UNIQUE_PUBKEY_CAPACITY),
-            failed_inserts: VecDeque::new(),
+            pull_request_time: RwLock::new(LruCache::new(CRDS_UNIQUE_PUBKEY_CAPACITY)),
+            failed_inserts: RwLock::default(),
             crds_timeout: CRDS_GOSSIP_PULL_CRDS_TIMEOUT_MS,
             msg_timeout: CRDS_GOSSIP_PULL_MSG_TIMEOUT_MS,
-            num_pulls: 0,
+            num_pulls: AtomicUsize::default(),
         }
     }
 }
 impl CrdsGossipPull {
-    /// generate a random request
+    /// Generate a random request
     #[allow(clippy::too_many_arguments)]
     pub(crate) fn new_pull_request(
         &self,
         thread_pool: &ThreadPool,
-        crds: &Crds,
+        crds: &RwLock<Crds>,
         self_keypair: &Keypair,
         self_shred_version: u16,
         now: u64,
@@ -226,8 +228,8 @@ impl CrdsGossipPull {
         pings: &mut Vec<(SocketAddr, Ping)>,
         socket_addr_space: &SocketAddrSpace,
     ) -> Result<(ContactInfo, Vec<CrdsFilter>), CrdsGossipError> {
-        let (weights, peers): (Vec<_>, Vec<_>) = self
-            .pull_options(
+        let (weights, peers): (Vec<_>, Vec<_>) = {
+            self.pull_options(
                 crds,
                 &self_keypair.pubkey(),
                 self_shred_version,
@@ -237,7 +239,9 @@ impl CrdsGossipPull {
                 socket_addr_space,
             )
             .into_iter()
-            .unzip();
+            .map(|(weight, node, gossip_addr)| (weight, (node, gossip_addr)))
+            .unzip()
+        };
         if peers.is_empty() {
             return Err(CrdsGossipError::NoPeers);
         }
@@ -250,36 +254,46 @@ impl CrdsGossipPull {
             let mut ping_cache = ping_cache.lock().unwrap();
             let mut pingf = move || Ping::new_rand(&mut rng, self_keypair).ok();
             let now = Instant::now();
-            peers.find(|peer| {
-                let node = (peer.id, peer.gossip);
-                let (check, ping) = ping_cache.check(now, node, &mut pingf);
+            peers.find(|node| {
+                let (_, gossip_addr) = *node;
+                let (check, ping) = ping_cache.check(now, *node, &mut pingf);
                 if let Some(ping) = ping {
-                    pings.push((peer.gossip, ping));
+                    pings.push((gossip_addr, ping));
                 }
                 check
             })
         };
-        match peer {
-            None => Err(CrdsGossipError::NoPeers),
-            Some(peer) => {
-                let filters = self.build_crds_filters(thread_pool, crds, bloom_size);
-                Ok((peer.clone(), filters))
-            }
-        }
+        let peer = match peer {
+            None => return Err(CrdsGossipError::NoPeers),
+            Some((node, _gossip_addr)) => node,
+        };
+        let filters = self.build_crds_filters(thread_pool, crds, bloom_size);
+        let peer = match crds.read().unwrap().get::<&ContactInfo>(peer) {
+            None => return Err(CrdsGossipError::NoPeers),
+            Some(node) => node.clone(),
+        };
+        Ok((peer, filters))
     }
 
-    fn pull_options<'a>(
+    fn pull_options(
         &self,
-        crds: &'a Crds,
+        crds: &RwLock<Crds>,
         self_id: &Pubkey,
         self_shred_version: u16,
         now: u64,
         gossip_validators: Option<&HashSet<Pubkey>>,
         stakes: &HashMap<Pubkey, u64>,
         socket_addr_space: &SocketAddrSpace,
-    ) -> Vec<(u64, &'a ContactInfo)> {
+    ) -> Vec<(
+        u64,        // weight
+        Pubkey,     // node
+        SocketAddr, // gossip address
+    )> {
         let mut rng = rand::thread_rng();
         let active_cutoff = now.saturating_sub(PULL_ACTIVE_TIMEOUT_MS);
+        let pull_request_time = self.pull_request_time.read().unwrap();
+        // crds should be locked last after self.pull_request_time.
+        let crds = crds.read().unwrap();
         crds.get_nodes()
             .filter_map(|value| {
                 let info = value.value.contact_info().unwrap();
@@ -303,8 +317,7 @@ impl CrdsGossipPull {
             })
             .map(|item| {
                 let max_weight = f32::from(u16::max_value()) - 1.0;
-                let req_time: u64 = self
-                    .pull_request_time
+                let req_time: u64 = pull_request_time
                     .peek(&item.id)
                     .copied()
                     .unwrap_or_default();
@@ -313,24 +326,26 @@ impl CrdsGossipPull {
                 let weight = get_weight(max_weight, since, stake);
                 // Weights are bounded by max_weight defined above.
                 // So this type-cast should be safe.
-                ((weight * 100.0) as u64, item)
+                ((weight * 100.0) as u64, item.id, item.gossip)
             })
             .collect()
     }
 
-    /// time when a request to `from` was initiated
+    /// Time when a request to `from` was initiated.
+    ///
     /// This is used for weighted random selection during `new_pull_request`
     /// It's important to use the local nodes request creation time as the weight
     /// instead of the response received time otherwise failed nodes will increase their weight.
-    pub(crate) fn mark_pull_request_creation_time(&mut self, from: Pubkey, now: u64) {
-        self.pull_request_time.put(from, now);
+    pub(crate) fn mark_pull_request_creation_time(&self, from: Pubkey, now: u64) {
+        self.pull_request_time.write().unwrap().put(from, now);
     }
 
-    /// process a pull request
-    pub(crate) fn process_pull_requests<I>(crds: &mut Crds, callers: I, now: u64)
+    /// Process a pull request
+    pub(crate) fn process_pull_requests<I>(crds: &RwLock<Crds>, callers: I, now: u64)
     where
         I: IntoIterator<Item = CrdsValue>,
     {
+        let mut crds = crds.write().unwrap();
         for caller in callers {
             let key = caller.pubkey();
             let _ = crds.insert(caller, now, GossipRoute::PullRequest);
@@ -341,7 +356,7 @@ impl CrdsGossipPull {
     /// Create gossip responses to pull requests
     pub(crate) fn generate_pull_responses(
         thread_pool: &ThreadPool,
-        crds: &Crds,
+        crds: &RwLock<Crds>,
         requests: &[(CrdsValue, CrdsFilter)],
         output_size_limit: usize, // Limit number of crds values returned.
         now: u64,
@@ -357,7 +372,7 @@ impl CrdsGossipPull {
     //  .2 => hash value of outdated values which will fail to insert.
     pub(crate) fn filter_pull_responses(
         &self,
-        crds: &Crds,
+        crds: &RwLock<Crds>,
         timeouts: &HashMap<Pubkey, u64>,
         responses: Vec<CrdsValue>,
         now: u64,
@@ -369,6 +384,7 @@ impl CrdsGossipPull {
             .get(&Pubkey::default())
             .copied()
             .unwrap_or(self.msg_timeout);
+        let crds = crds.read().unwrap();
         let upsert = |response: CrdsValue| {
             let owner = response.label().pubkey();
             // Check if the crds value is older than the msg_timeout
@@ -381,7 +397,7 @@ impl CrdsGossipPull {
             } else if now <= response.wallclock().saturating_add(timeout) {
                 active_values.push(response);
                 None
-            } else if crds.get_contact_info(owner).is_some() {
+            } else if crds.get::<&ContactInfo>(owner).is_some() {
                 // Silently insert this old value without bumping record
                 // timestamps
                 expired_values.push(response);
@@ -400,10 +416,10 @@ impl CrdsGossipPull {
         (active_values, expired_values, failed_inserts)
     }
 
-    /// process a vec of pull responses
+    /// Process a vec of pull responses
     pub(crate) fn process_pull_responses(
-        &mut self,
-        crds: &mut Crds,
+        &self,
+        crds: &RwLock<Crds>,
         from: &Pubkey,
         responses: Vec<CrdsValue>,
         responses_expired_timeout: Vec<CrdsValue>,
@@ -412,45 +428,53 @@ impl CrdsGossipPull {
         stats: &mut ProcessPullStats,
     ) {
         let mut owners = HashSet::new();
+        let mut crds = crds.write().unwrap();
         for response in responses_expired_timeout {
             let _ = crds.insert(response, now, GossipRoute::PullResponse);
         }
+        let mut num_inserts = 0;
         for response in responses {
             let owner = response.pubkey();
             if let Ok(()) = crds.insert(response, now, GossipRoute::PullResponse) {
-                stats.success += 1;
-                self.num_pulls += 1;
+                num_inserts += 1;
                 owners.insert(owner);
             }
         }
+        stats.success += num_inserts;
+        self.num_pulls.fetch_add(num_inserts, Ordering::Relaxed);
         owners.insert(*from);
         for owner in owners {
             crds.update_record_timestamp(&owner, now);
         }
+        drop(crds);
         stats.failed_insert += failed_inserts.len();
         self.purge_failed_inserts(now);
-        self.failed_inserts
-            .extend(failed_inserts.into_iter().zip(std::iter::repeat(now)));
+        let failed_inserts = failed_inserts.into_iter().zip(repeat(now));
+        self.failed_inserts.write().unwrap().extend(failed_inserts);
     }
 
-    pub(crate) fn purge_failed_inserts(&mut self, now: u64) {
+    pub(crate) fn purge_failed_inserts(&self, now: u64) {
         if FAILED_INSERTS_RETENTION_MS < now {
             let cutoff = now - FAILED_INSERTS_RETENTION_MS;
-            let outdated = self
-                .failed_inserts
+            let mut failed_inserts = self.failed_inserts.write().unwrap();
+            let outdated = failed_inserts
                 .iter()
                 .take_while(|(_, ts)| *ts < cutoff)
                 .count();
-            self.failed_inserts.drain(..outdated);
+            failed_inserts.drain(..outdated);
         }
     }
 
+    pub(crate) fn failed_inserts_size(&self) -> usize {
+        self.failed_inserts.read().unwrap().len()
+    }
+
     // build a set of filters of the current crds table
     // num_filters - used to increase the likelyhood of a value in crds being added to some filter
     pub fn build_crds_filters(
         &self,
         thread_pool: &ThreadPool,
-        crds: &Crds,
+        crds: &RwLock<Crds>,
         bloom_size: usize,
     ) -> Vec<CrdsFilter> {
         const PAR_MIN_LENGTH: usize = 512;
@@ -458,7 +482,10 @@ impl CrdsGossipPull {
         const MIN_NUM_BLOOM_ITEMS: usize = 512;
         #[cfg(not(debug_assertions))]
         const MIN_NUM_BLOOM_ITEMS: usize = 65_536;
-        let num_items = crds.len() + crds.num_purged() + self.failed_inserts.len();
+        let failed_inserts = self.failed_inserts.read().unwrap();
+        // crds should be locked last after self.failed_inserts.
+        let crds = crds.read().unwrap();
+        let num_items = crds.len() + crds.num_purged() + failed_inserts.len();
         let num_items = MIN_NUM_BLOOM_ITEMS.max(num_items);
         let filters = CrdsFilterSet::new(num_items, bloom_size);
         thread_pool.install(|| {
@@ -467,20 +494,22 @@ impl CrdsGossipPull {
                 .map(|v| v.value_hash)
                 .chain(crds.purged().with_min_len(PAR_MIN_LENGTH))
                 .chain(
-                    self.failed_inserts
+                    failed_inserts
                         .par_iter()
                         .with_min_len(PAR_MIN_LENGTH)
                         .map(|(v, _)| *v),
                 )
                 .for_each(|v| filters.add(v));
         });
+        drop(crds);
+        drop(failed_inserts);
         filters.into()
     }
 
-    /// filter values that fail the bloom filter up to max_bytes
+    /// Filter values that fail the bloom filter up to `max_bytes`.
     fn filter_crds_values(
         thread_pool: &ThreadPool,
-        crds: &Crds,
+        crds: &RwLock<Crds>,
         filters: &[(CrdsValue, CrdsFilter)],
         output_size_limit: usize, // Limit number of crds values returned.
         now: u64,
@@ -494,6 +523,7 @@ impl CrdsGossipPull {
         let total_skipped = AtomicUsize::default();
         let output_size_limit = output_size_limit.try_into().unwrap_or(i64::MAX);
         let output_size_limit = AtomicI64::new(output_size_limit);
+        let crds = crds.read().unwrap();
         let apply_filter = |caller: &CrdsValue, filter: &CrdsFilter| {
             if output_size_limit.load(Ordering::Relaxed) <= 0 {
                 return Vec::default();
@@ -566,10 +596,11 @@ impl CrdsGossipPull {
     /// Purge values from the crds that are older then `active_timeout`
     pub(crate) fn purge_active(
         thread_pool: &ThreadPool,
-        crds: &mut Crds,
+        crds: &RwLock<Crds>,
         now: u64,
         timeouts: &HashMap<Pubkey, u64>,
     ) -> usize {
+        let mut crds = crds.write().unwrap();
         let labels = crds.find_old_labels(thread_pool, now, timeouts);
         for label in &labels {
             crds.remove(label, now);
@@ -580,8 +611,8 @@ impl CrdsGossipPull {
     /// For legacy tests
     #[cfg(test)]
     fn process_pull_response(
-        &mut self,
-        crds: &mut Crds,
+        &self,
+        crds: &RwLock<Crds>,
         from: &Pubkey,
         timeouts: &HashMap<Pubkey, u64>,
         response: Vec<CrdsValue>,
@@ -608,17 +639,29 @@ impl CrdsGossipPull {
 
     // Only for tests and simulations.
     pub(crate) fn mock_clone(&self) -> Self {
-        let mut pull_request_time = LruCache::new(self.pull_request_time.cap());
-        for (k, v) in self.pull_request_time.iter().rev() {
-            pull_request_time.put(*k, *v);
-        }
+        let pull_request_time = {
+            let pull_request_time = self.pull_request_time.read().unwrap();
+            let mut clone = LruCache::new(pull_request_time.cap());
+            for (k, v) in pull_request_time.iter().rev() {
+                clone.put(*k, *v);
+            }
+            clone
+        };
+        let failed_inserts = self.failed_inserts.read().unwrap().clone();
         Self {
-            pull_request_time,
-            failed_inserts: self.failed_inserts.clone(),
+            pull_request_time: RwLock::new(pull_request_time),
+            failed_inserts: RwLock::new(failed_inserts),
+            num_pulls: AtomicUsize::new(self.num_pulls.load(Ordering::Relaxed)),
             ..*self
         }
     }
+
+    #[cfg(test)]
+    pub(crate) fn pull_request_time(&self) -> std::sync::RwLockReadGuard<LruCache<Pubkey, u64>> {
+        self.pull_request_time.read().unwrap()
+    }
 }
+
 #[cfg(test)]
 pub(crate) mod tests {
     use {
@@ -702,6 +745,7 @@ pub(crate) mod tests {
             stakes.insert(id, i * 100);
         }
         let now = 1024;
+        let crds = RwLock::new(crds);
         let mut options = node.pull_options(
             &crds,
             &me.label().pubkey(),
@@ -712,12 +756,10 @@ pub(crate) mod tests {
             &SocketAddrSpace::Unspecified,
         );
         assert!(!options.is_empty());
-        options.sort_by(|(weight_l, _), (weight_r, _)| weight_r.partial_cmp(weight_l).unwrap());
+        options
+            .sort_by(|(weight_l, _, _), (weight_r, _, _)| weight_r.partial_cmp(weight_l).unwrap());
         // check that the highest stake holder is also the heaviest weighted.
-        assert_eq!(
-            *stakes.get(&options.get(0).unwrap().1.id).unwrap(),
-            3000_u64
-        );
+        assert_eq!(stakes[&options[0].1], 3000_u64);
     }
 
     #[test]
@@ -761,6 +803,7 @@ pub(crate) mod tests {
             .unwrap();
         crds.insert(node_456.clone(), 0, GossipRoute::LocalMessage)
             .unwrap();
+        let crds = RwLock::new(crds);
 
         // shred version 123 should ignore nodes with versions 0 and 456
         let options = node
@@ -774,7 +817,7 @@ pub(crate) mod tests {
                 &SocketAddrSpace::Unspecified,
             )
             .iter()
-            .map(|(_, c)| c.id)
+            .map(|(_, pk, _)| *pk)
             .collect::<Vec<_>>();
         assert_eq!(options.len(), 1);
         assert!(!options.contains(&spy.pubkey()));
@@ -792,7 +835,7 @@ pub(crate) mod tests {
                 &SocketAddrSpace::Unspecified,
             )
             .iter()
-            .map(|(_, c)| c.id)
+            .map(|(_, pk, _)| *pk)
             .collect::<Vec<_>>();
         assert_eq!(options.len(), 3);
         assert!(options.contains(&me.pubkey()));
@@ -822,6 +865,7 @@ pub(crate) mod tests {
             .unwrap();
         crds.insert(node_123.clone(), 0, GossipRoute::LocalMessage)
             .unwrap();
+        let crds = RwLock::new(crds);
 
         // Empty gossip_validators -- will pull from nobody
         let mut gossip_validators = HashSet::new();
@@ -861,7 +905,7 @@ pub(crate) mod tests {
             &SocketAddrSpace::Unspecified,
         );
         assert_eq!(options.len(), 1);
-        assert_eq!(options[0].1.id, node_123.pubkey());
+        assert_eq!(options[0].1, node_123.pubkey());
     }
 
     #[test]
@@ -933,9 +977,11 @@ pub(crate) mod tests {
                 num_inserts += 1;
             }
         }
+        let crds = RwLock::new(crds);
         assert!(num_inserts > 30_000, "num inserts: {}", num_inserts);
         let filters = crds_gossip_pull.build_crds_filters(&thread_pool, &crds, MAX_BLOOM_SIZE);
         assert_eq!(filters.len(), MIN_NUM_BLOOM_FILTERS.max(32));
+        let crds = crds.read().unwrap();
         let purged: Vec<_> = thread_pool.install(|| crds.purged().collect());
         let hash_values: Vec<_> = crds.values().map(|v| v.value_hash).chain(purged).collect();
         // CrdsValue::new_rand may generate exact same value twice in which
@@ -965,13 +1011,13 @@ pub(crate) mod tests {
     #[test]
     fn test_new_pull_request() {
         let thread_pool = ThreadPoolBuilder::new().build().unwrap();
-        let mut crds = Crds::default();
+        let crds = RwLock::<Crds>::default();
         let node_keypair = Keypair::new();
         let entry = CrdsValue::new_unsigned(CrdsData::ContactInfo(ContactInfo::new_localhost(
             &node_keypair.pubkey(),
             0,
         )));
-        let mut node = CrdsGossipPull::default();
+        let node = CrdsGossipPull::default();
         let mut pings = Vec::new();
         let ping_cache = Mutex::new(PingCache::new(
             Duration::from_secs(20 * 60), // ttl
@@ -994,7 +1040,10 @@ pub(crate) mod tests {
             Err(CrdsGossipError::NoPeers)
         );
 
-        crds.insert(entry, 0, GossipRoute::LocalMessage).unwrap();
+        crds.write()
+            .unwrap()
+            .insert(entry, 0, GossipRoute::LocalMessage)
+            .unwrap();
         assert_eq!(
             node.new_pull_request(
                 &thread_pool,
@@ -1018,7 +1067,9 @@ pub(crate) mod tests {
             .unwrap()
             .mock_pong(new.id, new.gossip, Instant::now());
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
-        crds.insert(new.clone(), now, GossipRoute::LocalMessage)
+        crds.write()
+            .unwrap()
+            .insert(new.clone(), now, GossipRoute::LocalMessage)
             .unwrap();
         let req = node.new_pull_request(
             &thread_pool,
@@ -1039,7 +1090,9 @@ pub(crate) mod tests {
         node.mark_pull_request_creation_time(new.contact_info().unwrap().id, now);
         let offline = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), now);
         let offline = CrdsValue::new_unsigned(CrdsData::ContactInfo(offline));
-        crds.insert(offline, now, GossipRoute::LocalMessage)
+        crds.write()
+            .unwrap()
+            .insert(offline, now, GossipRoute::LocalMessage)
             .unwrap();
         let req = node.new_pull_request(
             &thread_pool,
@@ -1074,7 +1127,7 @@ pub(crate) mod tests {
             &node_keypair.pubkey(),
             0,
         )));
-        let mut node = CrdsGossipPull::default();
+        let node = CrdsGossipPull::default();
         crds.insert(entry, now, GossipRoute::LocalMessage).unwrap();
         let old = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), 0);
         ping_cache.mock_pong(old.id, old.gossip, Instant::now());
@@ -1086,6 +1139,7 @@ pub(crate) mod tests {
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
         crds.insert(new.clone(), now, GossipRoute::LocalMessage)
             .unwrap();
+        let crds = RwLock::new(crds);
 
         // set request creation time to now.
         let now = now + 50_000;
@@ -1125,7 +1179,7 @@ pub(crate) mod tests {
         const NUM_REPS: usize = 2 * CRDS_UNIQUE_PUBKEY_CAPACITY;
         let mut rng = rand::thread_rng();
         let pubkeys: Vec<_> = repeat_with(Pubkey::new_unique).take(NUM_REPS).collect();
-        let mut node = CrdsGossipPull::default();
+        let node = CrdsGossipPull::default();
         let mut requests = HashMap::new();
         let now = timestamp();
         for k in 0..NUM_REPS {
@@ -1134,21 +1188,22 @@ pub(crate) mod tests {
             node.mark_pull_request_creation_time(pubkey, now);
             *requests.entry(pubkey).or_default() = now;
         }
-        assert!(node.pull_request_time.len() <= CRDS_UNIQUE_PUBKEY_CAPACITY);
+        let pull_request_time = node.pull_request_time.read().unwrap();
+        assert!(pull_request_time.len() <= CRDS_UNIQUE_PUBKEY_CAPACITY);
         // Assert that timestamps match most recent request.
-        for (pk, ts) in &node.pull_request_time {
+        for (pk, ts) in pull_request_time.iter() {
             assert_eq!(*ts, requests[pk]);
         }
         // Assert that most recent pull timestamps are maintained.
         let max_ts = requests
             .iter()
-            .filter(|(pk, _)| !node.pull_request_time.contains(*pk))
+            .filter(|(pk, _)| !pull_request_time.contains(*pk))
             .map(|(_, ts)| *ts)
             .max()
             .unwrap();
         let min_ts = requests
             .iter()
-            .filter(|(pk, _)| node.pull_request_time.contains(*pk))
+            .filter(|(pk, _)| pull_request_time.contains(*pk))
             .map(|(_, ts)| *ts)
             .min()
             .unwrap();
@@ -1177,6 +1232,7 @@ pub(crate) mod tests {
         ping_cache.mock_pong(new.id, new.gossip, Instant::now());
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
         node_crds.insert(new, 0, GossipRoute::LocalMessage).unwrap();
+        let node_crds = RwLock::new(node_crds);
         let mut pings = Vec::new();
         let req = node.new_pull_request(
             &thread_pool,
@@ -1192,7 +1248,7 @@ pub(crate) mod tests {
             &SocketAddrSpace::Unspecified,
         );
 
-        let mut dest_crds = Crds::default();
+        let dest_crds = RwLock::<Crds>::default();
         let (_, filters) = req.unwrap();
         let mut filters: Vec<_> = filters.into_iter().map(|f| (caller.clone(), f)).collect();
         let rsp = CrdsGossipPull::generate_pull_responses(
@@ -1210,6 +1266,8 @@ pub(crate) mod tests {
             CRDS_GOSSIP_PULL_MSG_TIMEOUT_MS,
         )));
         dest_crds
+            .write()
+            .unwrap()
             .insert(
                 new,
                 CRDS_GOSSIP_PULL_MSG_TIMEOUT_MS,
@@ -1274,6 +1332,7 @@ pub(crate) mod tests {
         ping_cache.mock_pong(new.id, new.gossip, Instant::now());
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
         node_crds.insert(new, 0, GossipRoute::LocalMessage).unwrap();
+        let node_crds = RwLock::new(node_crds);
         let mut pings = Vec::new();
         let req = node.new_pull_request(
             &thread_pool,
@@ -1289,7 +1348,7 @@ pub(crate) mod tests {
             &SocketAddrSpace::Unspecified,
         );
 
-        let mut dest_crds = Crds::default();
+        let dest_crds = RwLock::<Crds>::default();
         let (_, filters) = req.unwrap();
         let filters: Vec<_> = filters.into_iter().map(|f| (caller.clone(), f)).collect();
         let rsp = CrdsGossipPull::generate_pull_responses(
@@ -1299,14 +1358,15 @@ pub(crate) mod tests {
             usize::MAX, // output_size_limit
             0,          // now
         );
-        CrdsGossipPull::process_pull_requests(
-            &mut dest_crds,
-            filters.into_iter().map(|(caller, _)| caller),
-            1,
-        );
+        let callers = filters.into_iter().map(|(caller, _)| caller);
+        CrdsGossipPull::process_pull_requests(&dest_crds, callers, 1);
+        let dest_crds = dest_crds.read().unwrap();
         assert!(rsp.iter().all(|rsp| rsp.is_empty()));
-        assert!(dest_crds.get(&caller.label()).is_some());
-        assert_eq!(dest_crds.get(&caller.label()).unwrap().local_timestamp, 1);
+        assert!(dest_crds.get::<&CrdsValue>(&caller.label()).is_some());
+        assert_eq!(1, {
+            let entry: &VersionedCrdsValue = dest_crds.get(&caller.label()).unwrap();
+            entry.local_timestamp
+        });
     }
     #[test]
     fn test_process_pull_request_response() {
@@ -1319,7 +1379,7 @@ pub(crate) mod tests {
         )));
         let caller = entry.clone();
         let node_pubkey = entry.label().pubkey();
-        let mut node = CrdsGossipPull::default();
+        let node = CrdsGossipPull::default();
         node_crds
             .insert(entry, 0, GossipRoute::LocalMessage)
             .unwrap();
@@ -1340,6 +1400,7 @@ pub(crate) mod tests {
         dest_crds
             .insert(new.clone(), 0, GossipRoute::LocalMessage)
             .unwrap();
+        let dest_crds = RwLock::new(dest_crds);
 
         // node contains a key from the dest node, but at an older local timestamp
         let same_key = ContactInfo::new_localhost(&new_id, 0);
@@ -1350,7 +1411,11 @@ pub(crate) mod tests {
         node_crds
             .insert(same_key.clone(), 0, GossipRoute::LocalMessage)
             .unwrap();
-        assert_eq!(node_crds.get(&same_key.label()).unwrap().local_timestamp, 0);
+        assert_eq!(0, {
+            let entry: &VersionedCrdsValue = node_crds.get(&same_key.label()).unwrap();
+            entry.local_timestamp
+        });
+        let node_crds = RwLock::new(node_crds);
         let mut done = false;
         let mut pings = Vec::new();
         let ping_cache = Mutex::new(ping_cache);
@@ -1379,7 +1444,7 @@ pub(crate) mod tests {
                 0,          // now
             );
             CrdsGossipPull::process_pull_requests(
-                &mut dest_crds,
+                &dest_crds,
                 filters.into_iter().map(|(caller, _)| caller),
                 0,
             );
@@ -1395,7 +1460,7 @@ pub(crate) mod tests {
             assert_eq!(rsp.len(), MIN_NUM_BLOOM_FILTERS);
             let failed = node
                 .process_pull_response(
-                    &mut node_crds,
+                    &node_crds,
                     &node_pubkey,
                     &node.make_timeouts(node_pubkey, &HashMap::new(), Duration::default()),
                     rsp.into_iter().flatten().collect(),
@@ -1403,9 +1468,17 @@ pub(crate) mod tests {
                 )
                 .0;
             assert_eq!(failed, 0);
-            assert_eq!(node_crds.get(&new.label()).unwrap().local_timestamp, 1);
+            assert_eq!(1, {
+                let node_crds = node_crds.read().unwrap();
+                let entry: &VersionedCrdsValue = node_crds.get(&new.label()).unwrap();
+                entry.local_timestamp
+            });
             // verify that the whole record was updated for dest since this is a response from dest
-            assert_eq!(node_crds.get(&same_key.label()).unwrap().local_timestamp, 1);
+            assert_eq!(1, {
+                let node_crds = node_crds.read().unwrap();
+                let entry: &VersionedCrdsValue = node_crds.get(&same_key.label()).unwrap();
+                entry.local_timestamp
+            });
             done = true;
             break;
         }
@@ -1432,24 +1505,30 @@ pub(crate) mod tests {
         node_crds
             .insert(old.clone(), 0, GossipRoute::LocalMessage)
             .unwrap();
-        let value_hash = node_crds.get(&old.label()).unwrap().value_hash;
-
+        let value_hash = {
+            let entry: &VersionedCrdsValue = node_crds.get(&old.label()).unwrap();
+            entry.value_hash
+        };
         //verify self is valid
         assert_eq!(
-            node_crds.get(&node_label).unwrap().value.label(),
+            node_crds.get::<&CrdsValue>(&node_label).unwrap().label(),
             node_label
         );
         // purge
+        let node_crds = RwLock::new(node_crds);
         let timeouts = node.make_timeouts(node_pubkey, &HashMap::new(), Duration::default());
-        CrdsGossipPull::purge_active(&thread_pool, &mut node_crds, node.crds_timeout, &timeouts);
+        CrdsGossipPull::purge_active(&thread_pool, &node_crds, node.crds_timeout, &timeouts);
 
         //verify self is still valid after purge
+        assert_eq!(node_label, {
+            let node_crds = node_crds.read().unwrap();
+            node_crds.get::<&CrdsValue>(&node_label).unwrap().label()
+        });
         assert_eq!(
-            node_crds.get(&node_label).unwrap().value.label(),
-            node_label
+            node_crds.read().unwrap().get::<&CrdsValue>(&old.label()),
+            None
         );
-        assert_eq!(node_crds.get(&old.label()), None);
-        assert_eq!(node_crds.num_purged(), 1);
+        assert_eq!(node_crds.read().unwrap().num_purged(), 1);
         for _ in 0..30 {
             // there is a chance of a false positive with bloom filters
             // assert that purged value is still in the set
@@ -1459,6 +1538,7 @@ pub(crate) mod tests {
         }
 
         // purge the value
+        let mut node_crds = node_crds.write().unwrap();
         node_crds.trim_purged(node.crds_timeout + 1);
         assert_eq!(node_crds.num_purged(), 0);
     }
@@ -1543,8 +1623,8 @@ pub(crate) mod tests {
 
     #[test]
     fn test_process_pull_response() {
-        let mut node_crds = Crds::default();
-        let mut node = CrdsGossipPull::default();
+        let node_crds = RwLock::<Crds>::default();
+        let node = CrdsGossipPull::default();
 
         let peer_pubkey = solana_sdk::pubkey::new_rand();
         let peer_entry = CrdsValue::new_unsigned(CrdsData::ContactInfo(
@@ -1556,7 +1636,7 @@ pub(crate) mod tests {
         // inserting a fresh value should be fine.
         assert_eq!(
             node.process_pull_response(
-                &mut node_crds,
+                &node_crds,
                 &peer_pubkey,
                 &timeouts,
                 vec![peer_entry.clone()],
@@ -1566,14 +1646,14 @@ pub(crate) mod tests {
             0
         );
 
-        let mut node_crds = Crds::default();
+        let node_crds = RwLock::<Crds>::default();
         let unstaked_peer_entry = CrdsValue::new_unsigned(CrdsData::ContactInfo(
             ContactInfo::new_localhost(&peer_pubkey, 0),
         ));
         // check that old contact infos fail if they are too old, regardless of "timeouts"
         assert_eq!(
             node.process_pull_response(
-                &mut node_crds,
+                &node_crds,
                 &peer_pubkey,
                 &timeouts,
                 vec![peer_entry.clone(), unstaked_peer_entry],
@@ -1583,11 +1663,11 @@ pub(crate) mod tests {
             4
         );
 
-        let mut node_crds = Crds::default();
+        let node_crds = RwLock::<Crds>::default();
         // check that old contact infos can still land as long as they have a "timeouts" entry
         assert_eq!(
             node.process_pull_response(
-                &mut node_crds,
+                &node_crds,
                 &peer_pubkey,
                 &timeouts,
                 vec![peer_entry],
@@ -1604,7 +1684,7 @@ pub(crate) mod tests {
         // but a recent contact info (inserted above) exists
         assert_eq!(
             node.process_pull_response(
-                &mut node_crds,
+                &node_crds,
                 &peer_pubkey,
                 &timeouts,
                 vec![peer_vote.clone()],
@@ -1614,11 +1694,11 @@ pub(crate) mod tests {
             0
         );
 
-        let mut node_crds = Crds::default();
+        let node_crds = RwLock::<Crds>::default();
         // without a contact info, inserting an old value should fail
         assert_eq!(
             node.process_pull_response(
-                &mut node_crds,
+                &node_crds,
                 &peer_pubkey,
                 &timeouts,
                 vec![peer_vote],
